'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: device.py
@time: 2020-03-31 18:21:21
@desc: 
'''
import torch

device = torch.device('cuda') if torch.cuda.is_available() else torch.device('cpu')


'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: logger.py
@time: 2020-03-31 17:33:44
@desc: 
'''
import os
import torch
import json


class Logger:
    def __init__(self, output=str):
        self.output = output
        self.__init_folders__()

    def __init_folders__(self):
        self.train_log_path = os.path.join(self.output, "train_logs")
        self.eval_log_path = os.path.join(self.output, "eval_logs")
        self.test_log_path = os.path.join(self.output, 'test_logs')

        folders = [self.output,
                   self.train_log_path,
                   self.eval_log_path,
                   self.test_log_path]

        for folder in folders:
            if not os.path.exists(folder):
                os.mkdir(folder)

        if not os.path.exists(os.path.join(self.train_log_path, "checkpoint.txt")):
            with open(os.path.join(self.train_log_path, "checkpoint.txt"), 'w') as file:
                file.close()

        if not os.path.exists(os.path.join(self.train_log_path, "loss.txt")):
            with open(os.path.join(self.train_log_path, "loss.txt"), 'w') as file:
                file.close()

        if not os.path.exists(os.path.join(self.eval_log_path, "eval.txt")):
            with open(os.path.join(self.eval_log_path, "eval.txt"), 'w') as file:
                file.close()

        self.checkpoint_file = os.path.join(self.train_log_path, "checkpoint.txt")
        self.loss_file = os.path.join(self.train_log_path, "loss.txt")
        self.eval_file = os.path.join(self.eval_log_path, "eval.txt")

    def logger(self, content, phase='c'):
        '''
        :param content:
        :param phase:c => checkpoint_file; l => loss_file; e => eval_file
        :return:
        '''
        f = None
        if phase == 'c':
            f = self.checkpoint_file
        elif phase == 'l':
            f = self.loss_file
        elif phase == 'e':
            f = self.eval_file
        else:
            raise Exception('phase must be c or l or e')

        with open(f, 'a') as file:
            file.writelines(content + '\n')

    def save_checkpoints_file(self, epoch, state_dict=dict):
        pth_file_name = os.path.join(self.train_log_path, '%d.pth' % epoch )
        torch.save(state_dict, pth_file_name)
        self.logger(content='epoch {epoch:d}: {filename}'.format(epoch=epoch, filename=pth_file_name),phase='c')

    def save_eval_json_file(self, epoch, jsonContent):
        with open(os.path.join(self.eval_log_path, "%d.json" % epoch), 'w') as f:
            f.write(json.dumps(jsonContent))

    def get_path_files(self):
        return self.train_log_path, self.eval_log_path, self.test_log_path, self.checkpoint_file, self.loss_file, self.eval_file



'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: imageutils.py
@time: 2019-09-06 10:33:48
@desc: 
'''
import PIL.Image as Image
import cv2
import numpy as np
import os
import imageio


class CV2ToPIL:
    def __init__(self):
        pass

    def __call__(self, image):
        image = Image.fromarray(cv2.cvtColor(image,cv2.COLOR_BGR2RGB))
        return image


class PILToCV2:
    def __init__(self):
        pass

    def __call__(self, image):
        image = cv2.cvtColor(np.asarray(image),cv2.COLOR_RGB2BGR)
        return image


class ImageToGif:
    def __init__(self):
        pass

    def __call__(self, imagefolder, duration=0.1, **kwargs):
        filenames = sorted(
            (os.path.join(imagefolder, fn) for fn in os.listdir(imagefolder) if fn.endswith('.png') or fn.endswith('.jpg')))
        images = []
        for filename in filenames:
            images.append(imageio.imread(filename))
        imageio.mimsave('{}.gif'.format(filename[:filename.rfind('.')]), images, duration=duration)


'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: dataset.py
@time: 2020-03-27 15:24:07
@desc: 
'''
import os
from jjzhk.config import ZKCFG
import tqdm
import cv2
import xml.etree.ElementTree as ET
import pycocotools.coco as coco
import zipfile
import numpy as np


class DataSetBase(object):
    def __init__(self, root,imageset, isSegmentation):
        self._root_path = root
        self._imageset = imageset
        self._img_list = []
        self._isSegmentation = isSegmentation
        self.__prepare__()

    def __len__(self):
        return len(self._img_list)

    def __getitem__(self, index):
        image = self.getImage(index)
        label = self.getLabel(index)
        if self._isSegmentation:
            mask = self.getMask(index)
            return image, label, mask

        return image, label

    def __prepare__(self):
        self.prepare()

    def __getItemInfo__(self, index):

        return self._img_list[index]

    def __getItemInfoByImageId__(self, imgid):
        pass

    def __getIndexByImageId__(self, imgid):
        index = 0
        for item in self._img_list:
            if item["img_id"] == imgid:
                return index

            index += 1

        return -1

    def prepare(self):
        pass

    def getImage(self, index):
        pass

    def getLabel(self, index):
        pass

    def getMask(self, index):
        pass


class VOCDataSet(DataSetBase):
    def __init__(self, config:ZKCFG, imageset, isSegmentation):
        root = os.path.join(config.BASE.DATA_ROOT, config.BASE.DATA_SUB_ROOT)
        self.dataConfig = config
        super(VOCDataSet, self).__init__(root=root, imageset=imageset, isSegmentation=isSegmentation)

    def prepare(self):
        prefix = "seg" if self._isSegmentation else "det"
        image_list_file = os.path.join(self._root_path, "MainSet", "%s_%s.txt" % (prefix, self._imageset))
        lines = [x.strip() for x in open(image_list_file, 'r').readlines()]

        bar = tqdm.tqdm(lines)
        for line in bar:
            bar.set_description("Processing %s" % self._imageset)
            l = self.__getItemInfoByImageId__(line)
            self._img_list.append(l)

    def getImage(self, index):
        fname = os.path.join(self._root_path, "JPEGImages", "%s.jpg" % self._img_list[index]["img_id"])
        self._img_list[index]["path"] = fname
        return cv2.imread(fname)

    def getLabel(self, index):
        return self._img_list[index]["boxes"]

    def getMask(self, index):
        info = self._img_list[index]
        file_index = info["img_id"]
        lbl_path = os.path.join(self._root_path, 'SegmentationDecode', file_index + '.png')
        label_mask = cv2.imread(lbl_path, cv2.IMREAD_GRAYSCALE)

        return label_mask

    '''
    {
        img_id : "***",
        width : "***",
        height : "***",
        boxes : [[classs_id, xmin, ymin, xmax, ymax],[classs_id, xmin, ymin, xmax, ymax]
        detail : [{"name":***,"pose":***, "truncated":***,"difficult":***,},{....},{....}]
        ...]
    }
    '''
    def __getItemInfoByImageId__(self, image_id):
        image_info = {}
        image_info["img_id"] = image_id
        labels = []
        detail = []
        anno_file = os.path.join(self._root_path, "Annotations", "%s.xml" % image_id)
        root = ET.parse(anno_file).getroot()
        size = root.find('size')
        w = int(size.find('width').text)
        h = int(size.find('height').text)
        image_info["width"] = w
        image_info["height"] = h

        for obj in root.iter('object'):
            class_name = obj.find('name').text
            xmlbox = obj.find('bndbox')
            left   = int(xmlbox.find('xmin').text)
            right  = int(xmlbox.find('xmax').text)
            top    = int(xmlbox.find('ymin').text)
            bottom = int(xmlbox.find('ymax').text)
            class_id = self.dataConfig.classid(class_name)
            item = [class_id, left, top, right, bottom]
            labels.append(item)

            pose = obj.find("pose").text
            truncated = int(obj.find("truncated").text)
            difficult = int(obj.find("difficult").text)
            detail.append({"name": class_name, "pose":pose, "truncated" : truncated, "difficult" : difficult})

        image_info["boxes"] = labels
        image_info["detail"] = detail

        return image_info


class COCODataSet(DataSetBase):
    def __init__(self, config:ZKCFG, imageset, isSegmentation):
        self.dataConfig = config
        self.year = 2017
        super(COCODataSet, self).__init__(root=os.path.join(config.BASE.DATA_ROOT, config.BASE.DATA_SUB_ROOT), imageset=imageset, isSegmentation=isSegmentation)
        prefix = "%s%d" % (self._imageset, self.year)
        self.zip = zipfile.ZipFile(os.path.join(self._root_path, "%s.zip" % (prefix) ))

    def prepare(self):
        prefix = "%s%d" % (self._imageset, self.year)
        annFile = os.path.join(self._root_path, "annotations", "instances_%s.json" % prefix)
        print(annFile)
        self.coco = coco.COCO(annFile)
        class_ids = sorted(self.coco.getCatIds()) # all classes

        lines = []
        for id in class_ids:
            lines.extend(list(self.coco.getImgIds(catIds=[id])))
        # Remove duplicates
        lines = list(set(lines))

        bar = tqdm.tqdm(lines)
        for line in bar:
            bar.set_description("Processing %s" % self._imageset)
            l = self.__getItemInfoByImageId__(line)
            self._img_list.append(l)

    def getImage(self, index):
        info = self._img_list[index]
        img_info = self.coco.imgs[info["img_id"]]
        self._img_list[index]["path"] = ""
        prefix = "%s%d" % (self._imageset, self.year)
        r = self.zip.read('%s/%s' % (prefix, img_info['file_name']))
        arr = cv2.imdecode(np.frombuffer(r, np.uint8), 1)

        return arr

    def getLabel(self, index):
        return self._img_list[index]["boxes"]

    def getMask(self, index):
        info = self._img_list[index]
        img_id = info['img_id']
        height = info['height']
        width = info['width']
        annIds = self.coco.getAnnIds(imgIds=img_id, iscrowd=None)
        anns = self.coco.loadAnns(annIds)
        MASK = np.zeros((height, width), dtype=np.uint16)

        for i, ann in enumerate(anns):
            mask = self.coco.annToMask(ann)
            idxs = np.where(mask > 0)
            MASK[idxs] = ann['category_id']

        return MASK

    '''
    {
        img_id : "***",
        width : "***",
        height : "***",
        boxes : [[classs_id, xmin, ymin, xmax, ymax],[classs_id, xmin, ymin, xmax, ymax]
        ...]
    }
    '''
    def __getItemInfoByImageId__(self, image_id):
        image_info = self.coco.imgs[image_id]
        img_info_json = {}
        img_info_json["img_id"] = image_id
        img_info_json["width"] = image_info["width"]
        img_info_json["height"] = image_info["height"]
        class_coco = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 27, 28,
                      31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 46, 47, 48, 49, 50, 51, 52, 53, 54,
                      55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 67, 70, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
                      82, 84, 85, 86, 87, 88, 89, 90]
        annIds = self.coco.getAnnIds(imgIds=image_id, iscrowd=None)
        ann_info = self.coco.loadAnns(annIds)
        item = []
        for ann in ann_info:
            class_id = [class_coco.index(int(ann["category_id"]))]
            bbox = ann["bbox"]
            bbox[2] = bbox[0] + bbox[2]
            bbox[3] = bbox[1] + bbox[3]
            item.append(class_id + bbox + [1 if ann['iscrowd'] == 1 else 0])
        img_info_json["boxes"] = item
        img_info_json["anno"] = ann_info
        return img_info_json


class TestImageDataSet(DataSetBase):
    def __init__(self, cfg:ZKCFG, isSegmentation=False):
        self.cfg = cfg
        super(TestImageDataSet, self).__init__(root=cfg.BASE.TEST_DATA_ROOT, imageset='test', isSegmentation=isSegmentation)

    def prepare(self):
        image_list_file = os.path.join(self._root_path, "info.txt")
        lines = [x.strip() for x in open(image_list_file, 'r').readlines()]

        bar = tqdm.tqdm(lines)
        for line in bar:
            bar.set_description("Processing %s" % self._imageset)
            self._img_list.append({"img_id" : line})

    def getImage(self, index):
        fname = os.path.join(self._root_path, "Images", "%s.jpg" % self._img_list[index]["img_id"])
        image = cv2.imread(fname)
        self._img_list[index]["path"] = fname
        self._img_list[index]["height"] = image.shape[0]
        self._img_list[index]["width"] = image.shape[1]
        return image

    def getLabel(self, index):
        return None

    def getMask(self, index):
        return None


'''
@author: JJZHK
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: Config.py
@time: 2018/12/11 10:18
@desc: 
'''
import os
import yaml
import json


class YamlConfig(object):
    def __init__(self, config_dict:dict):
        for key, val in config_dict.items():
            if isinstance(val, dict):
                self.__setattr__(key, YamlConfig(val))
            elif isinstance(val, list):
                self.__setattr__(key, [])
                for obj in val:
                    if isinstance(obj, dict):
                        self[key].append(YamlConfig(obj))
                    else:
                        self[key].append(obj)
            else:
                self.__setattr__(key, val)

    def copy(self, new_config_dict={}):
        ret = YamlConfig(vars(self))
        for key, val in new_config_dict.items():
            ret.__setattr__(key, val)

        return ret

    def replace(self, new_config_dict):
        for key in new_config_dict.keys():
            val = new_config_dict[key]
            if isinstance(val, YamlConfig):
                if hasattr(self, key):
                    self[key].replace(val)
                else:
                    self.__setattr__(key, val)
            else:
                self.__setattr__(key, val)

    def __getitem__(self, key):
        return self.__getattribute__(key)

    def keys(self):
        return list(vars(self).keys())

    def values(self):
        return list(vars(self).values())

    def print(self):
        for k, v in vars(self).items():
            print(k, ' = ', v)


class ZKCFG:
    def __init__(self,basefile:str, cfgfile:str, rootpath='cfgs'):
        self._base_file_ = None
        self._cfg_file_ = None
        self.CFGData = None

        if basefile is not None:
            with open(os.path.join(rootpath, basefile), 'r') as file:
                data = yaml.load(file,Loader=yaml.FullLoader)
                self._base_file_ = YamlConfig(data)

        if cfgfile is not None:
            with open(os.path.join(rootpath, cfgfile), 'r') as file:
                data = yaml.load(file,Loader=yaml.FullLoader)
                self._cfg_file_ = YamlConfig(data)

        assert self._base_file_ is not None or self._cfg_file_ is not None

        if self._base_file_ is None:
            self.CFGData = self._cfg_file_
        elif self._cfg_file_ is None:
            self.CFGData = self._base_file_
        else:
            self._base_file_.replace(self._cfg_file_)
            self.CFGData = self._base_file_

    def __getattr__(self, name):
        return self.CFGData[name]

    def __getitem__(self, key):
        return self.CFGData[key]

    def color(self, classname:str):
        return self.CFGData.BASE.CLASSINFO[classname]

    def classname(self,idx:int):
        return self.CFGData.BASE.CLASSINFO.keys()[idx]

    def classid(self, classname:str):
        return self.CFGData.BASE.CLASSINFO.keys().index(classname)

    def print(self):
        self.CFGData.print()
# class Config:
#     def __init__(self, cfgfile, commonfile=None, cfgroot="cfgs/", loadtype='f'):
#         '''
#         :param commonfile: 公共部分的文件地址或字符串
#         :param cfgfile: 需要加载的yml文件名或字符串
#         :param cfgroot: yml目录地址
#         :param loadtype: 加载方式：f-yml文件加载，s-字符串加载
#         '''
#         self.commonfile = commonfile
#         self.loadtype = loadtype
#         self.cfgroot = cfgroot
#         self.CFGData = AttrDict({})
#         if self.loadtype == 'f':
#             with open(os.path.join(cfgroot, cfgfile), 'r') as file:
#                 data = yaml.load(file, Loader=yaml.FullLoader)
#         elif self.loadtype == 's':
#             with open(os.path.join(cfgfile), 'r') as file:
#                 data = yaml.load(file,Loader=yaml.FullLoader)
#         else:
#             raise Exception("Load type is error.")
#
#         data = json.loads(json.dumps(data))
#         self.CFGData = AttrDict(data)
#         if self.commonfile is not None:
#             self._initcfg_()
#
#         self._mergecfg_(self.CFGData, AttrDict(data))
#
#         self._updatecfg_()
#
#     def _initcfg_(self):
#         if self.loadtype == 'f':
#             with open(os.path.join(self.cfgroot, self.commonfile), 'r') as file:
#                 data = yaml.load(file, Loader=yaml.FullLoader)
#         elif self.loadtype == 's':
#             data = yaml.load(self.commonfile, Loader=yaml.FullLoader)
#         else:
#             raise Exception("Load type is error.")
#
#         data = json.loads(json.dumps(data))
#         self.CFGData = AttrDict(data)
#         self._initspecial_()
#
#     def _mergecfg_(self,target, data):
#         target.merge(data)
#
#     def _updatecfg_(self):
#         pass
#
#     def _initspecial_(self):
#         pass


# class DataConfig:
#     def __init__(self, cfg=Config):
#         self.cfg = cfg
#         self.classinfo = self.cfg.BASE.CLASSINFO
#
#         self.colors = list(self.classinfo.todict().values())
#         self.classes = list(self.classinfo.todict().keys())
#         if self.classes[0] == 'background':
#             self.classes.remove('background')
#
#         self.classes_num = len(self.classes)
#         self.dataset = None
#         self.name = self.cfg.BASE.DATA_TYPE
#
#     def getClasses(self):
#
#         return self.classes
#
#     def getColors(self):
#         return self.colors
#
#     def getNumByClass(self, cls):
#         classes = self.getClasses()
#         return classes.index(cls)
#
#     def getClassByNum(self, num):
#         classes = self.getClasses()
#         return classes[num]


# if __name__ == '__main__':
#     cfg = ZKCFG(basefile='basic_config_voc.yml', cfgfile="vgg16.yml", rootpath='')
#     # cfg.T = "CC"
#     # print(cfg.T)



import sys
import time, datetime


class Timer(object):
    def __init__(self):
        self.init_time = time.time()
        self.total_time = 0
        self.calls = 0
        self.start_time = 0
        self.diff = 0
        self.average_time = 0
        self.remain_time = 0

    def tic(self):
        self.start_time = time.time()

    def toc(self, average=True):
        self.diff = time.time() - self.start_time
        self.total_time += self.diff
        self.calls += 1
        self.average_time = self.total_time / self.calls
        if average:
            return self.average_time
        else:
            return self.diff

    def remain(self, iters, max_iters):
        if iters == 0:
            self.remain_time = 0
        else:
            self.remain_time = (time.time() - self.init_time) * (max_iters - iters) / iters

        return str(datetime.timedelta(seconds=int(self.remain_time)))

    def average(self):
        return str("%.3f" % self.average_time)

    def averagetostr(self):
        return str(datetime.timedelta(seconds=int(time.time() - self.start_time))), int(time.time() - self.start_time)


class ProgressBar():
    def __init__(self, epoch_count, one_batch_count, pattern, **kwargs):
        self.total_count = one_batch_count
        self.current_index = 0
        self.current_epoch = 1
        self.epoch_count = epoch_count
        self.train_timer = Timer()
        self.pattern = pattern
        self.iter = False
        if 'iter' in kwargs:
            self.iter = True
        self.showIndex = 0

    def show(self,currentEpoch, *args):
        self.showIndex += 1
        if self.iter == True and self.showIndex == 1:
            self.current_index = self.epoch_count
        else:
            self.current_index += 1
        if self.showIndex == 1:
            self.train_timer.tic()
        self.current_epoch = currentEpoch
        perCount = int(self.total_count / 100) # 7
        perCount = 1 if perCount == 0 else perCount
        percent = int(self.current_index / perCount)

        if self.total_count % perCount == 0:
            dotcount = int(self.total_count / perCount)
        else:
            dotcount = int(self.total_count / perCount)

        s1 = "\rEpoch:%d / %d [%s%s] %d / %d "%(
            self.current_epoch,
            self.epoch_count if self.iter == False else 1,
            "*"*(int(percent)),
            " "*(dotcount-int(percent)),
            self.current_index,
            self.total_count
        )

        s2 = self.pattern % tuple([float("{:f}".format(x)) for x in args])

        s3 = "%s,%s,remain=%s" % (
            s1, s2, self.train_timer.remain(self.current_index, self.total_count))
        sys.stdout.write(s3)
        sys.stdout.flush()
        if self.current_index == self.total_count :
            total_time_str, total_time = self.train_timer.averagetostr()
            self.train_timer.toc()
            s3 = "%s,%s,total=%s" % (
                s1, s2, total_time_str)
            sys.stdout.write(s3)
            sys.stdout.flush()
            self.current_index = 0
            self.showIndex = 0
            print("\r")
            return total_time

        return 0


'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: __init__.py
@time: 2020-03-27 14:04:32
@desc: 
'''
from jjzhk.drawseg import BaseDrawSeg
from jjzhk.imageutils import CV2ToPIL, PILToCV2, ImageToGif
from jjzhk.progressbar import ProgressBar
from jjzhk.config import ZKCFG, YamlConfig
from jjzhk.device import device
from jjzhk.logger import Logger


'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: setup.py
@time: 2019-09-06 10:31:15
@desc: 
'''
from setuptools import setup, find_packages

setup(
    name = 'pyjjzhktools',
    version = '1.5.4',
    keywords='jjzhk tools',
    description = 'a library',
    license = 'MIT License',
    url = 'https://gitlab.com/jjzhk-group/tools/pyjjzhktools',
    author = 'Jeff K Zhang',
    author_email = 'jeffcobile@gmail.com',
    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = ['PyYAML'],
)


'''
@author: zhangkai
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: drawseg.py
@time: 2020-03-27 13:42:54
@desc: 
'''
import numpy as np
import PIL.ImageDraw as pil_imagedraw
import PIL.ImageFont as pil_imagefont
import PIL.Image as Image
import jjzhk.imageutils as eui
from jjzhk.config import ZKCFG
import cv2
import os


class BaseDrawSeg:
    def __init__(self,cfg:ZKCFG, output=str):
        '''
        :param classinfo:
        {
            "classlabel" : "color" # the first node should be background
        }
        :param output: path which images to save
        '''
        self.cfg = cfg
        self.output = output
        if not os.path.exists(self.output):
            os.mkdir(self.output)

    def drawByImage(self, image, boxes, mask, imagename):
        image_type = type(image)
        if image_type == np.ndarray:
            img = Image.fromarray(np.uint8(image * 255))
        elif image_type == str:
            image = cv2.imread(image)
            cv2topil = eui.CV2ToPIL()
            img = cv2topil(image)
        elif image_type == Image.Image:
            img = image
        else:
            raise ValueError("drawByImage just support str,ndarray and PIL.Image types")

        return self._draw_(img, boxes, mask, imagename)

    def _draw_(self, image, boxes, mask, imagename):
        img, mask = self._draw_box_(image, boxes, mask)

        img.save(os.path.join(self.output, "%s.jpg" % imagename))
        if mask is not None:
            mask.save(os.path.join(self.output, "%s_mask.jpg" % imagename))

    def _draw_box_(self, image, boxes, mask):
        font = pil_imagefont.truetype(font='Courier.dfont',
                                      size=np.floor(3e-2 * image.size[1] + 0.5).astype('int32'))
        thickness = (image.size[0] + image.size[1]) // 300
        if boxes is not None:
            for (x1, y1), (x2, y2), class_name, _, prob in boxes:
                draw = pil_imagedraw.Draw(image)
                left = int(x1)
                top = int(y1)
                right = int(x2)
                bottom = int(y2)
                color = tuple(self.cfg.color(class_name))

                label = '{} {:.2f}'.format(class_name, prob)
                label_size = draw.textsize(label, font)

                for i in range(thickness):
                    draw.rectangle(
                        [left + i, top + i, right - i, bottom - i], outline=color)

                if top - label_size[1] >= 0:
                    text_origin = np.array([left, top - label_size[1]])
                else:
                    text_origin = np.array([left, top + 1])

                draw.rectangle(
                    [tuple(text_origin), tuple(text_origin + label_size)],
                    fill=color)
                draw.text(text_origin, label, fill=(255, 255, 255), font=font)
                del draw

        if mask is not None:
            mask = cv2.cvtColor(self._maskToImg_(mask), cv2.COLOR_RGB2BGR)
            piltocv2 = eui.PILToCV2()
            image = piltocv2(image)
            image = cv2.addWeighted(image, 0.6, mask, 0.4, 0)
            cv2topil = eui.CV2ToPIL()
            image = cv2topil(image)
            mask = cv2topil(mask)
        return image, mask

    def _maskToImg_(self, mask):
        r = mask.copy()
        g = mask.copy()
        b = mask.copy()
        CLASS = self.cfg.BASE.CLASSINFO.keys()
        for ll, classname in enumerate(CLASS):
            if ll == 0:
                continue
            r[mask == ll] = self.cfg.color(classname)[0]
            g[mask == ll] = self.cfg.color(classname)[1]
            b[mask == ll] = self.cfg.color(classname)[2]

        rgb = np.zeros((mask.shape[0], mask.shape[1], 3), dtype=np.uint8)
        rgb[:, :, 0] = r
        rgb[:, :, 1] = g
        rgb[:, :, 2] = b

        return rgb